@@ -43,7 +43,7 @@
 #include "audio_clock.h"
 #include "actions.h"
 #include "utils.h"
-#include "color_manager.h"
+#include "theme_manager.h"
 
 #include "i18n.h"
 
@@ -76,7 +76,7 @@ ARDOUR_UI::setup_windows ()
 	setup_transport();
 	build_menu_bar ();
 
-	color_manager->signal_unmap().connect (bind (sigc::ptr_fun(&ActionManager::uncheck_toggleaction), X_("<Actions>/Common/ToggleColorManager")));
+	theme_manager->signal_unmap().connect (bind (sigc::ptr_fun(&ActionManager::uncheck_toggleaction), X_("<Actions>/Common/ToggleThemeManager")));
 
 	top_packer.pack_start (menu_bar_base, false, false);
 	top_packer.pack_start (transport_frame, false, false);
@@ -271,8 +271,8 @@ ARDOUR_UI::setup_transport ()
 
 	/* clocks, etc. */
 
-	ARDOUR_UI::Clock.connect (bind (mem_fun (primary_clock, &AudioClock::set), false));
-	ARDOUR_UI::Clock.connect (bind (mem_fun (secondary_clock, &AudioClock::set), false));
+	ARDOUR_UI::Clock.connect (bind (mem_fun (primary_clock, &AudioClock::set), 1));
+	ARDOUR_UI::Clock.connect (bind (mem_fun (secondary_clock, &AudioClock::set), 2));
 
 	primary_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::primary_clock_value_changed));
 	secondary_clock.ValueChanged.connect (mem_fun(*this, &ARDOUR_UI::secondary_clock_value_changed));
