@@ -75,6 +75,12 @@ ARDOUR_UI::toggle_send_midi_feedback ()
 	ActionManager::toggle_config_state ("options", "SendMIDIfeedback", &Configuration::set_midi_feedback, &Configuration::get_midi_feedback);
 }
 
+void
+ARDOUR_UI::toggle_denormal_protection ()
+{
+	ActionManager::toggle_config_state ("options", "DenormalProtection", &Configuration::set_denormal_protection, &Configuration::get_denormal_protection);
+}
+
 void
 ARDOUR_UI::set_native_file_header_format (HeaderFormat hf)
 {
@@ -129,6 +135,9 @@ ARDOUR_UI::set_native_file_data_format (SampleFormat sf)
 	case FormatInt24:
 		action = X_("FileDataFormat24bit");
 		break;
+	case FormatInt16:
+		action = X_("FileDataFormat16bit");
+		break;
 	default:
 		fatal << string_compose (_("programming error: %1"), "illegal file data format in ::set_native_file_data_format") << endmsg;
 		/*NOTREACHED*/
@@ -292,6 +301,45 @@ ARDOUR_UI::set_monitor_model (MonitorModel model)
 
 }
 
+void
+ARDOUR_UI::set_denormal_model (DenormalModel model)
+{
+	const char* action = 0;
+
+	switch (model) {
+	case DenormalNone:
+		action = X_("DenormalNone");
+		break;
+
+	case DenormalFTZ:
+		action = X_("DenormalFTZ");
+		break;
+		
+	case DenormalDAZ:
+		action = X_("DenormalDAZ");
+		break;
+
+	case DenormalFTZDAZ:
+		action = X_("DenormalFTZDAZ");
+		break;
+
+	default:
+		fatal << string_compose (_("programming error: unknown denormal model in ARDOUR_UI::set_denormal_model: %1"), model) << endmsg;
+		/*NOTREACHED*/
+	}
+
+	Glib::RefPtr<Action> act = ActionManager::get_action ("options", action);
+
+	if (act) {
+		Glib::RefPtr<RadioAction> ract = Glib::RefPtr<RadioAction>::cast_dynamic(act);
+
+		if (ract && ract->get_active() && Config->get_denormal_model() != model) {
+			Config->set_denormal_model (model);
+		}
+	}
+
+}
+
 void
 ARDOUR_UI::toggle_auto_input ()
 {
@@ -434,6 +482,18 @@ ARDOUR_UI::toggle_ShowSoloMutes()
 	ActionManager::toggle_config_state ("options", "ShowSoloMutes", &Configuration::set_show_solo_mutes, &Configuration::get_show_solo_mutes);
 }
 
+void
+ARDOUR_UI::toggle_PrimaryClockDeltaEditCursor()
+{
+	ActionManager::toggle_config_state ("options", "PrimaryClockDeltaEditCursor", &Configuration::set_primary_clock_delta_edit_cursor, &Configuration::get_primary_clock_delta_edit_cursor);
+}
+
+void
+ARDOUR_UI::toggle_SecondaryClockDeltaEditCursor()
+{
+	ActionManager::toggle_config_state ("options", "SecondaryClockDeltaEditCursor", &Configuration::set_secondary_clock_delta_edit_cursor, &Configuration::get_secondary_clock_delta_edit_cursor);
+}
+
 void
 ARDOUR_UI::mtc_port_changed ()
 {
@@ -516,6 +576,36 @@ ARDOUR_UI::map_monitor_model ()
 	}
 }
 
+void
+ARDOUR_UI::map_denormal_model ()
+{
+	const char* on = 0;
+
+	switch (Config->get_denormal_model()) {
+	case DenormalNone:
+		on = X_("DenormalNone");
+		break;
+	case DenormalFTZ:
+		on = X_("DenormalFTZ");
+		break;
+	case DenormalDAZ:
+		on = X_("DenormalDAZ");
+		break;
+	case DenormalFTZDAZ:
+		on = X_("DenormalFTZDAZ");
+		break;
+	}
+
+	Glib::RefPtr<Action> act = ActionManager::get_action ("options", on);
+	if (act) {
+		Glib::RefPtr<ToggleAction> tact = Glib::RefPtr<ToggleAction>::cast_dynamic(act);
+
+		if (tact && !tact->get_active()) {
+			tact->set_active (true);
+		}
+	}
+}
+
 void
 ARDOUR_UI::map_remote_model ()
 {
@@ -605,6 +695,10 @@ ARDOUR_UI::map_file_data_format ()
 		action = X_("FileDataFormat24bit");
 		break;
 
+	case FormatInt16:
+		action = X_("FileDataFormat16bit");
+		break;
+
 	default:
 		fatal << string_compose (_("programming error: unknown file data format passed to ARDOUR_UI::map_file_data_format: %1"), 
 					 Config->get_native_file_data_format()) << endmsg;
@@ -905,6 +999,8 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 		ActionManager::map_some_state ("options",  "StopTransportAtEndOfSession", &Configuration::get_stop_at_session_end);
 	} else if (PARAM_IS ("monitoring-model")) {
 		map_monitor_model ();
+	} else if (PARAM_IS ("denormal-model")) {
+		map_denormal_model ();
 	} else if (PARAM_IS ("remote-model")) {
 		map_remote_model ();
 	} else if (PARAM_IS ("use-video-sync")) {
@@ -962,7 +1058,11 @@ ARDOUR_UI::parameter_changed (const char* parameter_name)
 		}
 	} else if (PARAM_IS ("use-overlap-equivalency")) {
 		ActionManager::map_some_state ("options", "RegionEquivalentsOverlap", &Configuration::get_use_overlap_equivalency);
-	}
+	} else if (PARAM_IS ("primary-clock-delta-edit-cursor")) {
+		ActionManager::map_some_state ("options",  "PrimaryClockDeltaEditCursor", &Configuration::get_primary_clock_delta_edit_cursor);
+	} else if (PARAM_IS ("secondary-clock-delta-edit-cursor")) {
+		ActionManager::map_some_state ("options",  "SecondaryClockDeltaEditCursor", &Configuration::get_secondary_clock_delta_edit_cursor);
+	} 
 			   
 
 #undef PARAM_IS
