@@ -48,7 +48,6 @@
 #include "canvas_impl.h"
 #include "simplerect.h"
 #include "waveview.h"
-#include "color.h"
 
 using namespace std;
 using namespace ARDOUR;
@@ -129,23 +128,23 @@ CrossfadeEditor::CrossfadeEditor (Session& s, boost::shared_ptr<Crossfade> xf, d
 	toplevel->property_x2() =  10.0;
 	toplevel->property_y2() =  10.0;
 	toplevel->property_fill() =  true;
-	toplevel->property_fill_color_rgba() = color_map[cCrossfadeEditorBase];
+	toplevel->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorBase.get();
 	toplevel->property_outline_pixels() =  0;
 	toplevel->signal_event().connect (mem_fun (*this, &CrossfadeEditor::canvas_event));
 	
 	fade[Out].line = new ArdourCanvas::Line (*(canvas->root()));
 	fade[Out].line->property_width_pixels() = 1;
-	fade[Out].line->property_fill_color_rgba() = color_map[cCrossfadeEditorLine];
+	fade[Out].line->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorLine.get();
 		
 	fade[Out].shading = new ArdourCanvas::Polygon (*(canvas->root()));
-	fade[Out].shading->property_fill_color_rgba() = color_map[cCrossfadeEditorLineShading];
+	fade[Out].shading->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorLineShading.get();
 
 	fade[In].line = new ArdourCanvas::Line (*(canvas->root()));
 	fade[In].line->property_width_pixels() = 1;
-	fade[In].line->property_fill_color_rgba() = color_map[cCrossfadeEditorLine];
+	fade[In].line->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorLine.get();
 		
 	fade[In].shading = new ArdourCanvas::Polygon (*(canvas->root()));
-	fade[In].shading->property_fill_color_rgba() = color_map[cCrossfadeEditorLineShading];
+	fade[In].shading->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorLineShading.get();
 	
 	fade[In].shading->signal_event().connect (mem_fun (*this, &CrossfadeEditor::canvas_event));
 	fade[In].line->signal_event().connect (mem_fun (*this, &CrossfadeEditor::curve_event));
@@ -454,8 +453,8 @@ CrossfadeEditor::make_point ()
 
 	p->box = new ArdourCanvas::SimpleRect (*(canvas->root()));
 	p->box->property_fill() = true;
-	p->box->property_fill_color_rgba() = color_map[cCrossfadeEditorPointFill];
-	p->box->property_outline_color_rgba() = color_map[cCrossfadeEditorPointOutline];
+	p->box->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorPointFill.get();
+	p->box->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorPointOutline.get();
 	p->box->property_outline_pixels() = 1;
 
 	p->curve = fade[current].line;
@@ -721,6 +720,20 @@ CrossfadeEditor::redraw ()
 void
 CrossfadeEditor::apply_preset (Preset *preset)
 {
+  
+	WhichFade wf =  find(fade_in_presets->begin(), fade_in_presets->end(), preset) != fade_in_presets->end() ? In : Out;
+	
+	if (current != wf) {
+	  
+	      	if (wf == In) {
+			select_in_button.clicked();
+		} else {
+			select_out_button.clicked();
+		}
+		
+		curve_select_clicked (wf);
+	}  
+	
 	for (list<Point*>::iterator i = fade[current].points.begin(); i != fade[current].points.end(); ++i) {
 		delete *i;
 	}
@@ -820,6 +833,8 @@ CrossfadeEditor::reset ()
 {
 	set (xfade->fade_in(),  In);
 	set (xfade->fade_out(), Out);
+
+        curve_select_clicked (current);
 }
 
 void
@@ -882,6 +897,15 @@ CrossfadeEditor::build_presets ()
 	p->push_back (PresetPoint (1, 1));
 	fade_in_presets->push_back (p);
 
+	p = new Preset ("crossfade_in_default");
+	p->push_back (PresetPoint (0, 0));
+	p->push_back (PresetPoint (0.1, 0.01));
+	p->push_back (PresetPoint (0.2, 0.03));
+	p->push_back (PresetPoint (0.8, 0.97));
+	p->push_back (PresetPoint (0.9, 0.99));
+	p->push_back (PresetPoint (1, 1));
+	fade_in_presets->push_back (p);
+	
 	/* FADE OUT */
 
 	// p = new Preset ("hiout.xpm");
@@ -932,6 +956,15 @@ CrossfadeEditor::build_presets ()
 	p->push_back (PresetPoint (0, 1));
 	p->push_back (PresetPoint (1, 0));
 	fade_out_presets->push_back (p);
+	
+	p = new Preset ("crossfade_out_default");
+	p->push_back (PresetPoint (0, 1));
+	p->push_back (PresetPoint (0.1, 0.99));
+	p->push_back (PresetPoint (0.2, 0.97));
+	p->push_back (PresetPoint (0.8, 0.03));
+	p->push_back (PresetPoint (0.9, 0.01));
+	p->push_back (PresetPoint (1, 0));
+	fade_out_presets->push_back (p);
 }
 
 void
@@ -942,15 +975,15 @@ CrossfadeEditor::curve_select_clicked (WhichFade wf)
 	if (wf == In) {
 		
 		for (vector<ArdourCanvas::WaveView*>::iterator i = fade[In].waves.begin(); i != fade[In].waves.end(); ++i) {
-			(*i)->property_wave_color() = color_map[cSelectedCrossfadeEditorWave];
+			(*i)->property_wave_color() = ARDOUR_UI::config()->canvasvar_SelectedCrossfadeEditorWave.get();
 		}
 
 		for (vector<ArdourCanvas::WaveView*>::iterator i = fade[Out].waves.begin(); i != fade[Out].waves.end(); ++i) {
-			(*i)->property_wave_color() = color_map[cCrossfadeEditorWave];
+			(*i)->property_wave_color() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorWave.get();
 		}
 
-		fade[In].line->property_fill_color_rgba() = color_map[cSelectedCrossfadeEditorLine];
-		fade[Out].line->property_fill_color_rgba() = color_map[cCrossfadeEditorLine];
+		fade[In].line->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_SelectedCrossfadeEditorLine.get();
+		fade[Out].line->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorLine.get();
 		fade[Out].shading->hide();
 		fade[In].shading->show();
 
@@ -962,26 +995,18 @@ CrossfadeEditor::curve_select_clicked (WhichFade wf)
 			(*i)->box->show ();
 		}
 
-		for (vector<Button*>::iterator i = fade_out_buttons.begin(); i != fade_out_buttons.end(); ++i) {
-			(*i)->set_sensitive (false);
-		}
-
-		for (vector<Button*>::iterator i = fade_in_buttons.begin(); i != fade_in_buttons.end(); ++i) {
-			(*i)->set_sensitive (true);
-		}
-
 	} else {
 
 		for (vector<ArdourCanvas::WaveView*>::iterator i = fade[In].waves.begin(); i != fade[In].waves.end(); ++i) {
-			(*i)->property_wave_color() = color_map[cCrossfadeEditorWave];
+			(*i)->property_wave_color() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorWave.get();
 		}
 
 		for (vector<ArdourCanvas::WaveView*>::iterator i = fade[Out].waves.begin(); i != fade[Out].waves.end(); ++i) {
-			(*i)->property_wave_color() = color_map[cSelectedCrossfadeEditorWave];
+			(*i)->property_wave_color() = ARDOUR_UI::config()->canvasvar_SelectedCrossfadeEditorWave.get();
 		}
 
-		fade[Out].line->property_fill_color_rgba() = color_map[cSelectedCrossfadeEditorLine];
-		fade[In].line->property_fill_color_rgba() = color_map[cCrossfadeEditorLine];
+		fade[Out].line->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_SelectedCrossfadeEditorLine.get();
+		fade[In].line->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_CrossfadeEditorLine.get();
 		fade[In].shading->hide();
 		fade[Out].shading->show();
 
@@ -993,14 +1018,6 @@ CrossfadeEditor::curve_select_clicked (WhichFade wf)
 			(*i)->box->show();
 		}
 
-		for (vector<Button*>::iterator i = fade_out_buttons.begin(); i != fade_out_buttons.end(); ++i) {
-			(*i)->set_sensitive (true);
-		}
-
-		for (vector<Button*>::iterator i = fade_in_buttons.begin(); i != fade_in_buttons.end(); ++i) {
-			(*i)->set_sensitive (false);
-		}
-
 	}
 }
 
@@ -1031,9 +1048,9 @@ CrossfadeEditor::make_waves (boost::shared_ptr<AudioRegion> region, WhichFade wh
 	double spu;
 
 	if (which == In) {
-		color = color_map[cSelectedCrossfadeEditorWave];
+		color = ARDOUR_UI::config()->canvasvar_SelectedCrossfadeEditorWave.get();
 	} else {
-		color = color_map[cCrossfadeEditorWave];
+		color = ARDOUR_UI::config()->canvasvar_CrossfadeEditorWave.get();
 	}
 
 	ht = canvas->get_allocation().get_height() / (double) nchans;
