@@ -26,6 +26,8 @@
 #include <string>
 #include <sys/time.h>
 
+#include <boost/optional.hpp>
+
 #include <libgnomecanvasmm/canvas.h>
 #include <libgnomecanvasmm/group.h>
 #include <libgnomecanvasmm/line.h>
@@ -349,6 +351,9 @@ class Editor : public PublicEditor
 	
 	ARDOUR::Session     *session;
 	bool                 constructed;
+  
+	// to keep track of the playhead position for control_scroll
+	boost::optional<nframes_t> _control_scroll_target;
 
 	PlaylistSelector* _playlist_selector;
 
@@ -1288,7 +1293,14 @@ class Editor : public PublicEditor
 	void tempo_map_changed (ARDOUR::Change);
 	void redisplay_tempo (bool immediate_redraw);
 	
-	void snap_to (nframes_t& first, int32_t direction = 0, bool for_mark = false);
+	void snap_to (nframes64_t& first, int32_t direction = 0, bool for_mark = false);
+	void snap_to (nframes_t& first, int32_t direction = 0, bool for_mark = false) {
+		/* XXX remove this function when everything moves to 64 bit frame counts */
+		nframes64_t first64 = first;
+		snap_to (first64, direction, for_mark);
+		first = (nframes_t) first64;
+	}
+
 	uint32_t bbt_beat_subdivision;
 
 	/* toolbar */
@@ -1844,6 +1856,7 @@ class Editor : public PublicEditor
 	Glib::RefPtr<Gtk::Action>              redo_action;
 
 	void history_changed ();
+	void color_handler ();
 };
 
 #endif /* __ardour_editor_h__ */
