@@ -193,7 +193,7 @@ RouteTimeAxisView::RouteTimeAxisView (PublicEditor& ed, Session& sess, boost::sh
 	}
 
 	editor.ZoomChanged.connect (mem_fun(*this, &RouteTimeAxisView::reset_samples_per_unit));
-	ColorChanged.connect (mem_fun (*this, &RouteTimeAxisView::color_handler));
+	ColorsChanged.connect (mem_fun (*this, &RouteTimeAxisView::color_handler));
 }
 
 RouteTimeAxisView::~RouteTimeAxisView ()
@@ -296,7 +296,6 @@ RouteTimeAxisView::set_edit_group_from_menu (RouteGroup *eg)
 
 void
 RouteTimeAxisView::playlist_changed ()
-
 {
 	label_view ();
 
@@ -595,8 +594,8 @@ RouteTimeAxisView::show_timestretch (nframes_t start, nframes_t end)
 		timestretch_rect->property_y1() =  0.0;
 		timestretch_rect->property_x2() =  0.0;
 		timestretch_rect->property_y2() =  0.0;
-		timestretch_rect->property_fill_color_rgba() =  color_map[cTimeStretchFill];
-		timestretch_rect->property_outline_color_rgba() = color_map[cTimeStretchOutline];
+		timestretch_rect->property_fill_color_rgba() =  ARDOUR_UI::config()->canvasvar_TimeStretchFill.get();
+		timestretch_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeStretchOutline.get();
 	}
 
 	timestretch_rect->show ();
@@ -1360,17 +1359,14 @@ RouteTimeAxisView::map_frozen ()
 }
 
 void
-RouteTimeAxisView::color_handler (ColorID id, uint32_t val)
+RouteTimeAxisView::color_handler ()
 {
-	switch (id) {
-	case cTimeStretchOutline:
-		timestretch_rect->property_outline_color_rgba() = val;
-		break;
-	case cTimeStretchFill:
-		timestretch_rect->property_fill_color_rgba() = val;
-		break;
-	default:
-		break;
+	if (timestretch_rect) {
+		timestretch_rect->property_outline_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeStretchOutline.get();
+	}
+
+	if (timestretch_rect) {
+		timestretch_rect->property_fill_color_rgba() = ARDOUR_UI::config()->canvasvar_TimeStretchFill.get();
 	}
 }
 
@@ -1542,7 +1538,7 @@ RouteTimeAxisView::add_redirect_automation_curve (boost::shared_ptr<Redirect> re
 					  *redirect, what, _session, *ran->view,
 					  *ran->view->canvas_display, redirect->automation_list (what));
 	
-	ral->set_line_color (color_map[cRedirectAutomationLine]);
+	ral->set_line_color (ARDOUR_UI::config()->canvasvar_RedirectAutomationLine.get());
 	ral->queue_reset ();
 
 	ran->view->add_line (*ral);
