@@ -343,6 +343,7 @@ class Session : public PBD::StatefulDestructible
 	sigc::signal<void,nframes_t> PositionChanged; /* sent after any non-sequential motion */
 	sigc::signal<void> DurationChanged;
 	sigc::signal<void> HaltOnXrun;
+	sigc::signal<void> TransportLooped;
 
 	sigc::signal<void,RouteList&> RouteAdded;
 
@@ -1497,7 +1498,7 @@ class Session : public PBD::StatefulDestructible
 	boost::shared_ptr<Playlist> XMLPlaylistFactory (const XMLNode&);
 
 	void playlist_length_changed ();
-	void diskstream_playlist_changed (boost::shared_ptr<Diskstream>);
+	void diskstream_playlist_changed (boost::weak_ptr<Diskstream>);
 
 	/* NAMED SELECTIONS */
 
