@@ -463,7 +463,7 @@ AudioDiskstream::check_record_status (nframes_t transport_frame, nframes_t nfram
 			
 		}
 
-		if (_flags & Recordable) {
+		if (recordable() && destructive()) {
 			boost::shared_ptr<ChannelList> c = channels.reader();
 			for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan) {
 				
@@ -555,6 +555,12 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 	if (nominally_recording || (_session.get_record_enabled() && Config->get_punch_in())) {
 		OverlapType ot;
 		
+		// Safeguard against situations where process() goes haywire when autopunching and last_recordable_frame < first_recordable_frame
+		if (last_recordable_frame < first_recordable_frame) {
+                        last_recordable_frame = max_frames;
+
+		}
+		
 		ot = coverage (first_recordable_frame, last_recordable_frame, transport_frame, transport_frame + nframes);
 
 		switch (ot) {
@@ -592,7 +598,7 @@ AudioDiskstream::process (nframes_t transport_frame, nframes_t nframes, nframes_
 			/*    |--------|    recrange
                             --------------  transrange
 			*/
-			rec_nframes = last_recordable_frame - last_recordable_frame;
+			rec_nframes = last_recordable_frame - first_recordable_frame;
 			rec_offset = first_recordable_frame - transport_frame;
 			break;
 		}
@@ -1001,6 +1007,8 @@ AudioDiskstream::read (Sample* buf, Sample* mixdown_buffer, float* gain_buffer,
 	nframes_t loop_start = 0;
 	nframes_t loop_length = 0;
 	nframes_t offset = 0;
+	nframes_t xfade_samples = 0;
+	Sample    xfade_buf[128];
 	Location *loc = 0;
 
 	/* XXX we don't currently play loops in reverse. not sure why */
@@ -1063,6 +1071,26 @@ AudioDiskstream::read (Sample* buf, Sample* mixdown_buffer, float* gain_buffer,
 			return -1;
 		}
 
+		// xfade loop boundary if appropriate
+
+		if (xfade_samples > 0) {
+			// just do a linear xfade for this short bit
+
+			xfade_samples = min(xfade_samples, this_read);
+
+			float delta = 1.0f / xfade_samples;
+			float scale = 0.0f;
+			Sample * tmpbuf = buf+offset;
+
+			for (size_t i=0; i < xfade_samples; ++i) {
+				*tmpbuf = (*tmpbuf * scale) + xfade_buf[i]*(1.0f - scale);
+				++tmpbuf;
+				scale += delta; 
+			}
+
+			xfade_samples = 0;
+		}
+
 		_read_data_count = _playlist->read_data_count();
 		
 		if (reversed) {
@@ -1070,13 +1098,22 @@ AudioDiskstream::read (Sample* buf, Sample* mixdown_buffer, float* gain_buffer,
 			swap_by_ptr (buf, buf + this_read - 1);
 			
 		} else {
-			
+			start += this_read;
+		
 			/* if we read to the end of the loop, go back to the beginning */
 			
 			if (reloop) {
+				// read crossfade samples to apply to the next read
+
+				xfade_samples = min((nframes_t) 128, cnt-this_read);
+
+				if (audio_playlist()->read (xfade_buf, mixdown_buffer, gain_buffer, start, xfade_samples, channel) != xfade_samples) {
+					error << string_compose(_("AudioDiskstream %1: cannot read xfade samples %2 from playlist at frame %3"), 
+								_id, xfade_samples,start) << endmsg;
+					memset(xfade_buf, 0, xfade_samples * sizeof(Sample)); // just in case
+				}
+
 				start = loop_start;
-			} else {
-				start += this_read;
 			}
 		} 
 
@@ -1631,6 +1668,57 @@ AudioDiskstream::transport_stopped (struct tm& when, time_t twhen, bool abort_ca
 	capture_start_frame = 0;
 }
 
+void
+AudioDiskstream::transport_looped (nframes_t transport_frame)
+{
+	if (was_recording) {
+		// all we need to do is finish this capture, with modified capture length
+		boost::shared_ptr<ChannelList> c = channels.reader();
+
+		// adjust the capture length knowing that the data will be recorded to disk
+		// only necessary after the first loop where we're recording
+		if (capture_info.size() == 0) {
+			capture_captured += _capture_offset;
+
+			if (_alignment_style == ExistingMaterial) {
+				capture_captured += _session.worst_output_latency();
+			} else {
+				capture_captured += _roll_delay;
+			}
+		}
+
+		finish_capture (true, c);
+
+		// the next region will start recording via the normal mechanism
+		// we'll set the start position to the current transport pos
+		// no latency adjustment or capture offset needs to be made, as that already happened the first time
+		capture_start_frame = transport_frame;
+		first_recordable_frame = transport_frame; // mild lie
+		last_recordable_frame = max_frames;
+		was_recording = true;
+
+		if (recordable() && destructive()) {
+			for (ChannelList::iterator chan = c->begin(); chan != c->end(); ++chan) {
+				
+				RingBufferNPT<CaptureTransition>::rw_vector transvec;
+				(*chan)->capture_transition_buf->get_write_vector(&transvec);
+				
+				if (transvec.len[0] > 0) {
+					transvec.buf[0]->type = CaptureStart;
+					transvec.buf[0]->capture_val = capture_start_frame;
+					(*chan)->capture_transition_buf->increment_write_ptr(1);
+				}
+				else {
+					// bad!
+					fatal << X_("programming error: capture_transition_buf is full on rec loop!  inconceivable!") 
+					      << endmsg;
+				}
+ 			}
+		}
+
+	}
+}
+
 void
 AudioDiskstream::finish_capture (bool rec_monitors_input, boost::shared_ptr<ChannelList> c)
 {
@@ -1646,7 +1734,6 @@ AudioDiskstream::finish_capture (bool rec_monitors_input, boost::shared_ptr<Chan
 			RingBufferNPT<CaptureTransition>::rw_vector transvec;
 			(*chan)->capture_transition_buf->get_write_vector(&transvec);
 			
-			
 			if (transvec.len[0] > 0) {
 				transvec.buf[0]->type = CaptureEnd;
 				transvec.buf[0]->capture_val = capture_captured;
@@ -2192,6 +2279,12 @@ AudioDiskstream::use_pending_capture_data (XMLNode& node)
 				continue;
 			}
 
+			// This protects sessions from errant CapturingSources in stored sessions
+			struct stat sbuf;
+			if (stat (prop->value().c_str(), &sbuf)) {
+				continue;
+			}
+
 			try {
 				fs = boost::dynamic_pointer_cast<AudioFileSource> (SourceFactory::createWritable (_session, prop->value(), false, _session.frame_rate()));
 			}
@@ -2339,7 +2432,7 @@ AudioDiskstream::ChannelInfo::ChannelInfo (nframes_t bufsize, nframes_t speed_si
 
 	playback_buf = new RingBufferNPT<Sample> (bufsize);
 	capture_buf = new RingBufferNPT<Sample> (bufsize);
-	capture_transition_buf = new RingBufferNPT<CaptureTransition> (128);
+	capture_transition_buf = new RingBufferNPT<CaptureTransition> (256);
 	
 	/* touch the ringbuffer buffers, which will cause
 	   them to be mapped into locked physical RAM if
