@@ -28,6 +28,8 @@
 #include <midi++/port.h>
 #include <midi++/fd_midiport.h>
 
+namespace MIDI {
+
 class ALSA_RawMidiPort : public MIDI::FD_MidiPort
 
 {
@@ -35,8 +37,16 @@ class ALSA_RawMidiPort : public MIDI::FD_MidiPort
 	ALSA_RawMidiPort (MIDI::PortRequest &req) 
 		: FD_MidiPort (req, "/dev/snd", "midi") {}
 	virtual ~ALSA_RawMidiPort () {}
+
+	static std::string typestring;
+
+  protected:
+	std::string get_typestring () const {
+		return typestring;
+	}
 };
 
+}
 
 #endif // __alsa_rawmidi_h__
 
