@@ -38,6 +38,7 @@
 #include <pbd/pthread_utils.h>
 #include <pbd/error.h>
 #include <pbd/memento_command.h>
+#include <pbd/convert.h>
 
 #include <ardour/route.h>
 #include <ardour/session.h>
@@ -98,15 +99,20 @@ MackieControlProtocol::MackieControlProtocol (Session& session)
 	, _polling( true )
 	, pfd( 0 )
 	, nfds( 0 )
+	, _jog_wheel( *this )
 {
-	//cout << "MackieControlProtocol::MackieControlProtocol" << endl;
+#ifdef DEBUG
+	cout << "MackieControlProtocol::MackieControlProtocol" << endl;
+#endif
 	// will start reading from ports, as soon as there are some
 	pthread_create_and_store (X_("mackie monitor"), &thread, 0, _monitor_work, this);
 }
 
 MackieControlProtocol::~MackieControlProtocol()
 {
-	//cout << "~MackieControlProtocol::MackieControlProtocol" << endl;
+#ifdef DEBUG
+	cout << "~MackieControlProtocol::MackieControlProtocol" << endl;
+#endif
 	try
 	{
 		close();
@@ -119,7 +125,9 @@ MackieControlProtocol::~MackieControlProtocol()
 	{
 		cout << "~MackieControlProtocol caught unknown" << endl;
 	}
-	//cout << "finished ~MackieControlProtocol::MackieControlProtocol" << endl;
+#ifdef DEBUG
+	cout << "finished ~MackieControlProtocol::MackieControlProtocol" << endl;
+#endif
 }
 
 Mackie::Surface & MackieControlProtocol::surface()
@@ -275,7 +283,9 @@ void MackieControlProtocol::switch_banks( int initial )
 		uint32_t end_pos = min( route_table.size(), sorted.size() );
 		Sorted::iterator it = sorted.begin() + _current_initial_bank;
 		Sorted::iterator end = sorted.begin() + _current_initial_bank + end_pos;
-		//cout << "switch to " << _current_initial_bank << ", " << end_pos << endl;
+#ifdef DEBUG
+		cout << "switch to " << _current_initial_bank << ", " << end_pos << endl;
+#endif
 		
 		// link routes to strips
 		uint32_t i = 0;
@@ -283,7 +293,9 @@ void MackieControlProtocol::switch_banks( int initial )
 		{
 			boost::shared_ptr<Route> route = *it;
 			Strip & strip = *surface().strips[i];
-			//cout << "remote id " << route->remote_control_id() << " connecting " << route->name() << " to " << strip.name() << " with port " << port_for_id(i) << endl;
+#ifdef DEBUG
+			cout << "remote id " << route->remote_control_id() << " connecting " << route->name() << " to " << strip.name() << " with port " << port_for_id(i) << endl;
+#endif
 			route_table[i] = route;
 			RouteSignal * rs = new RouteSignal( *route, *this, strip, port_for_id(i) );
 			route_signals.push_back( rs );
@@ -356,7 +368,7 @@ void MackieControlProtocol::zero_all()
 	}
 }
 
-int MackieControlProtocol::set_active (bool yn)
+int MackieControlProtocol::set_active( bool yn )
 {
 	if ( yn != _active )
 	{
@@ -466,7 +478,7 @@ bool MackieControlProtocol::handle_strip_button( Control & control, ButtonState
 	if ( control.name() == "fader_touch" )
 	{
 		state = bs == press;
-		control.strip().gain().touch( state );
+		control.strip().gain().in_use( state );
 	}
 	
 	return state;
@@ -474,23 +486,26 @@ bool MackieControlProtocol::handle_strip_button( Control & control, ButtonState
 
 void MackieControlProtocol::update_led( Mackie::Button & button, Mackie::LedState ls )
 {
-	MackiePort * port = 0;
-	if ( button.group().is_strip() )
+	if ( ls != none )
 	{
-		if ( button.group().is_master() )
+		MackiePort * port = 0;
+		if ( button.group().is_strip() )
 		{
-			port = &mcu_port();
+			if ( button.group().is_master() )
+			{
+				port = &mcu_port();
+			}
+			else
+			{
+				port = &port_for_id( dynamic_cast<const Strip&>( button.group() ).index() );
+			}
 		}
 		else
 		{
-			port = &port_for_id( dynamic_cast<const Strip&>( button.group() ).index() );
+			port = &mcu_port();
 		}
+		port->write( builder.build_led( button, ls ) );
 	}
-	else
-	{
-		port = &mcu_port();
-	}
-	if ( ls != none ) port->write( builder.build_led( button, ls ) );
 }
 
 void MackieControlProtocol::update_global_button( const string & name, LedState ls )
@@ -505,7 +520,7 @@ void MackieControlProtocol::update_global_button( const string & name, LedState
 #ifdef DEBUG
 		cout << "Button " << name << " not found" << endl;
 #endif
-		}
+	}
 }
 
 // send messages to surface to set controls to correct values
@@ -523,6 +538,13 @@ void MackieControlProtocol::update_surface()
 		// update strip from route
 		master_route_signal->notify_all();
 		
+		// turn off the led ring, for bcf emulation mode
+		if ( mcu_port().emulation() == MackiePort::bcf2000 )
+		{
+			Control & control = *surface().controls_by_name["jog"];
+			mcu_port().write( builder.build_led_ring( dynamic_cast<Pot &>( control ), off ) );
+		}
+		
 		// update global buttons and displays
 		notify_record_state_changed();
 		notify_transport_state_changed();
@@ -553,6 +575,10 @@ void MackieControlProtocol::connect_session_signals()
 
 void MackieControlProtocol::add_port( MIDI::Port & midi_port, int number )
 {
+#ifdef DEBUG
+		cout << "add port " << midi_port.name() << ", " << midi_port.device() << endl;
+#endif
+
 	MackiePort * sport = new MackiePort( *this, midi_port, number );
 	_ports.push_back( sport );
 
@@ -737,7 +763,9 @@ void* MackieControlProtocol::_monitor_work (void* arg)
 
 XMLNode & MackieControlProtocol::get_state()
 {
-	//cout << "MackieControlProtocol::get_state" << endl;
+#ifdef DEBUG
+	cout << "MackieControlProtocol::get_state" << endl;
+#endif
 	
 	// add name of protocol
 	XMLNode* node = new XMLNode( X_("Protocol") );
@@ -753,7 +781,9 @@ XMLNode & MackieControlProtocol::get_state()
 
 int MackieControlProtocol::set_state( const XMLNode & node )
 {
-	//cout << "MackieControlProtocol::set_state: active " << _active << endl;
+#ifdef DEBUG
+	cout << "MackieControlProtocol::set_state: active " << _active << endl;
+#endif
 	int retval = 0;
 	
 	// fetch current bank
@@ -780,7 +810,7 @@ int MackieControlProtocol::set_state( const XMLNode & node )
 
 void MackieControlProtocol::handle_control_event( SurfacePort & port, Control & control, const ControlState & state )
 {
-	uint32_t index = control.ordinal() - 1 + ( port.number() * port.strips() );
+	// find the route for the control, if there is one
 	boost::shared_ptr<Route> route;
 	if ( control.group().is_strip() )
 	{
@@ -788,10 +818,14 @@ void MackieControlProtocol::handle_control_event( SurfacePort & port, Control &
 		{
 			route = master_route();
 		}
-		else if ( index < route_table.size() )
-			route = route_table[index];
 		else
-			cerr << "Warning: index is " << index << " which is not in the route table, size: " << route_table.size() << endl;
+		{
+			uint32_t index = control.ordinal() - 1 + ( port.number() * port.strips() );
+			if ( index < route_table.size() )
+				route = route_table[index];
+			else
+				cerr << "Warning: index is " << index << " which is not in the route table, size: " << route_table.size() << endl;
+		}
 	}
 	
 	// This handles control element events from the surface
@@ -800,30 +834,17 @@ void MackieControlProtocol::handle_control_event( SurfacePort & port, Control &
 	switch ( control.type() )
 	{
 		case Control::type_fader:
-			if ( control.group().is_strip() )
+			// find the route in the route table for the id
+			// if the route isn't available, skip it
+			// at which point the fader should just reset itself
+			if ( route != 0 )
 			{
-				// find the route in the route table for the id
-				// if the route isn't available, skip it
-				// at which point the fader should just reset itself
-				if ( route != 0 )
-				{
-					route->set_gain( slider_position_to_gain( state.pos ), this );
-					
-					// must echo bytes back to slider now, because
-					// the notifier only works if the fader is not being
-					// touched. Which it is if we're getting input.
-					port.write( builder.build_fader( (Fader&)control, state.pos ) );
-				}
-			}
-			else
-			{
-				// master fader
-				boost::shared_ptr<Route> route = master_route();
-				if ( route )
-				{
-					route->set_gain( slider_position_to_gain( state.pos ), this );
-					port.write( builder.build_fader( (Fader&)control, state.pos ) );
-				}
+				route->set_gain( slider_position_to_gain( state.pos ), this );
+				
+				// must echo bytes back to slider now, because
+				// the notifier only works if the fader is not being
+				// touched. Which it is if we're getting input.
+				port.write( builder.build_fader( (Fader&)control, state.pos ) );
 			}
 			break;
 			
@@ -845,9 +866,10 @@ void MackieControlProtocol::handle_control_event( SurfacePort & port, Control &
 			else if ( control.group().is_master() )
 			{
 				// master fader touch
-				boost::shared_ptr<Route> route = master_route();
-				if ( route )
+				if ( route != 0 )
+				{
 					handle_strip_button( control, state.button_state, route );
+				}
 			}
 			else
 			{
@@ -862,14 +884,15 @@ void MackieControlProtocol::handle_control_event( SurfacePort & port, Control &
 			{
 				if ( route != 0 )
 				{
-					if ( route->panner().size() == 1 )
+					// pan for mono input routes, or stereo linked panners
+					if ( route->panner().size() == 1 || ( route->panner().size() == 2 && route->panner().linked() ) )
 					{
 						// assume pan for now
 						float xpos;
 						route->panner()[0]->get_effective_position (xpos);
 						
 						// calculate new value, and trim
-						xpos += state.delta;
+						xpos += state.delta * state.sign;
 						if ( xpos > 1.0 )
 							xpos = 1.0;
 						else if ( xpos < 0.0 )
@@ -886,29 +909,13 @@ void MackieControlProtocol::handle_control_event( SurfacePort & port, Control &
 			}
 			else
 			{
-				if ( control.name() == "jog" )
+				if ( control.is_jog() )
 				{
-					// TODO use current snap-to setting?
-					long delta = state.ticks * 1000;
-					nframes_t next = session->transport_frame() + delta;
-					if ( delta < 0 && session->transport_frame() < (nframes_t) abs( delta )  )
-					{
-						next = session->current_start_frame();
-					}
-					else if ( next > session->current_end_frame() )
-					{
-						next = session->current_end_frame();
-					}
-					
-					// doesn't work very well
-					session->request_locate( next, session->transport_rolling() );
-					
-					// turn off the led ring, for bcf emulation mode
-					port.write( builder.build_led_ring( dynamic_cast<Pot &>( control ), off ) );
+					_jog_wheel.jog_event( port, control, state );
 				}
 				else
 				{
-					cout << "external controller" << state.ticks << endl;
+					cout << "external controller" << state.ticks * state.sign << endl;
 				}
 			}
 			break;
@@ -969,7 +976,7 @@ void MackieControlProtocol::notify_gain_changed( RouteSignal * route_signal )
 	try
 	{
 		Fader & fader = route_signal->strip().gain();
-		if ( !fader.touch() )
+		if ( !fader.in_use() )
 		{
 			route_signal->port().write( builder.build_fader( fader, gain_to_slider_position( route_signal->route().effective_gain() ) ) );
 		}
@@ -984,7 +991,28 @@ void MackieControlProtocol::notify_name_changed( void *, RouteSignal * route_sig
 {
 	try
 	{
-		// TODO implement MackieControlProtocol::notify_name_changed
+		Strip & strip = route_signal->strip();
+		if ( !strip.is_master() )
+		{
+			string line1;
+			string fullname = route_signal->route().name();
+			
+			if ( fullname.length() <= 6 )
+			{
+				line1 = fullname;
+			}
+			else
+			{
+				line1 = PBD::short_version( fullname, 6 );
+			}
+			
+			route_signal->port().write_sysex(
+				builder.strip_display( strip, 0, line1 )
+			);
+			route_signal->port().write_sysex(
+				builder.strip_display_blank( strip, 1 )
+			);
+		}
 	}
 	catch( exception & e )
 	{
@@ -992,18 +1020,17 @@ void MackieControlProtocol::notify_name_changed( void *, RouteSignal * route_sig
 	}
 }
 
-// TODO deal with > 1 channel being panned
 void MackieControlProtocol::notify_panner_changed( RouteSignal * route_signal )
 {
 	try
 	{
 		Pot & pot = route_signal->strip().vpot();
-		
-		if ( route_signal->route().panner().size() == 1 )
+		const Panner & panner = route_signal->route().panner();
+		if ( panner.size() == 1 || ( panner.size() == 2 && panner.linked() ) )
 		{
 			float pos;
-			route_signal->route().panner()[0]->get_effective_position( pos);
-			route_signal->port().write( builder.build_led_ring( pot, ControlState( on, pos ) ) );
+			route_signal->route().panner()[0]->get_effective_position( pos );
+			route_signal->port().write( builder.build_led_ring( pot, ControlState( on, pos ), MackieMidiBuilder::midi_pot_mode_dot ) );
 		}
 		else
 		{
@@ -1054,10 +1081,28 @@ void MackieControlProtocol::poll_automation()
 LedState MackieControlProtocol::frm_left_press( Button & button )
 {
 	// can use first_mark_before/after as well
+	unsigned long elapsed = _frm_left_last.restart();
+	
 	Location * loc = session->locations()->first_location_before (
 		session->transport_frame()
 	);
-	if ( loc != 0 ) session->request_locate( loc->start(), session->transport_rolling() );
+	
+	// allow a quick double to go past a previous mark 
+	if ( elapsed < 500 && loc != 0)
+	{
+		Location * loc_two_back = session->locations()->first_location_before ( loc->start() );
+		if ( loc_two_back != 0 )
+		{
+			loc = loc_two_back;
+		}
+	}
+	
+	// move to the location, if it's valid
+	if ( loc != 0 )
+	{
+		session->request_locate( loc->start(), session->transport_rolling() );
+	}
+	
 	return on;
 }
 
@@ -1127,12 +1172,16 @@ LedState MackieControlProtocol::record_release( Button & button )
 
 LedState MackieControlProtocol::rewind_press( Button & button )
 {
-	session->request_transport_speed( -4.0 );
+	_jog_wheel.push( JogWheel::speed );
+	_jog_wheel.transport_direction( -1 );
+	session->request_transport_speed( -_jog_wheel.transport_speed() );
 	return on;
 }
 
 LedState MackieControlProtocol::rewind_release( Button & button )
 {
+	_jog_wheel.pop();
+	_jog_wheel.transport_direction( 0 );
 	if ( _transport_previously_rolling )
 		session->request_transport_speed( 1.0 );
 	else
@@ -1142,12 +1191,16 @@ LedState MackieControlProtocol::rewind_release( Button & button )
 
 LedState MackieControlProtocol::ffwd_press( Button & button )
 {
-	session->request_transport_speed( 4.0 );
+	_jog_wheel.push( JogWheel::speed );
+	_jog_wheel.transport_direction( 1 );
+	session->request_transport_speed( _jog_wheel.transport_speed() );
 	return on;
 }
 
 LedState MackieControlProtocol::ffwd_release( Button & button )
 {
+	_jog_wheel.pop();
+	_jog_wheel.transport_direction( 0 );
 	if ( _transport_previously_rolling )
 		session->request_transport_speed( 1.0 );
 	else
@@ -1155,6 +1208,87 @@ LedState MackieControlProtocol::ffwd_release( Button & button )
 	return off;
 }
 
+LedState MackieControlProtocol::loop_press( Button & button )
+{
+	session->request_play_loop( !session->get_play_loop() );
+	return on;
+}
+
+LedState MackieControlProtocol::loop_release( Button & button )
+{
+	return session->get_play_loop();
+}
+
+LedState MackieControlProtocol::punch_in_press( Button & button )
+{
+	bool state = !Config->get_punch_in();
+	Config->set_punch_in( state );
+	return state;
+}
+
+LedState MackieControlProtocol::punch_in_release( Button & button )
+{
+	return Config->get_punch_in();
+}
+
+LedState MackieControlProtocol::punch_out_press( Button & button )
+{
+	bool state = !Config->get_punch_out();
+	Config->set_punch_out( state );
+	return state;
+}
+
+LedState MackieControlProtocol::punch_out_release( Button & button )
+{
+	return Config->get_punch_out();
+}
+
+LedState MackieControlProtocol::home_press( Button & button )
+{
+	session->goto_start();
+	return on;
+}
+
+LedState MackieControlProtocol::home_release( Button & button )
+{
+	return off;
+}
+
+LedState MackieControlProtocol::end_press( Button & button )
+{
+	session->goto_end();
+	return on;
+}
+
+LedState MackieControlProtocol::end_release( Button & button )
+{
+	return off;
+}
+
+LedState MackieControlProtocol::clicking_press( Button & button )
+{
+	bool state = !Config->get_clicking();
+	Config->set_clicking( state );
+	return state;
+}
+
+LedState MackieControlProtocol::clicking_release( Button & button )
+{
+	return Config->get_clicking();
+}
+
+LedState MackieControlProtocol::global_solo_press( Button & button )
+{
+	bool state = !session->soloing();
+	session->set_all_solo ( state );
+	return state;
+}
+
+LedState MackieControlProtocol::global_solo_release( Button & button )
+{
+	return session->soloing();
+}
+
 ///////////////////////////////////////////
 // Session signals
 ///////////////////////////////////////////
@@ -1250,87 +1384,6 @@ void MackieControlProtocol::notify_transport_state_changed()
 	mcu_port().write( builder.build_led( *rec, record_release( *rec ) ) );
 }
 
-LedState MackieControlProtocol::loop_press( Button & button )
-{
-	session->request_play_loop( !session->get_play_loop() );
-	return on;
-}
-
-LedState MackieControlProtocol::loop_release( Button & button )
-{
-	return session->get_play_loop();
-}
-
-LedState MackieControlProtocol::punch_in_press( Button & button )
-{
-	bool state = !Config->get_punch_in();
-	Config->set_punch_in( state );
-	return state;
-}
-
-LedState MackieControlProtocol::punch_in_release( Button & button )
-{
-	return Config->get_punch_in();
-}
-
-LedState MackieControlProtocol::punch_out_press( Button & button )
-{
-	bool state = !Config->get_punch_out();
-	Config->set_punch_out( state );
-	return state;
-}
-
-LedState MackieControlProtocol::punch_out_release( Button & button )
-{
-	return Config->get_punch_out();
-}
-
-LedState MackieControlProtocol::home_press( Button & button )
-{
-	session->goto_start();
-	return on;
-}
-
-LedState MackieControlProtocol::home_release( Button & button )
-{
-	return off;
-}
-
-LedState MackieControlProtocol::end_press( Button & button )
-{
-	session->goto_end();
-	return on;
-}
-
-LedState MackieControlProtocol::end_release( Button & button )
-{
-	return off;
-}
-
-LedState MackieControlProtocol::clicking_press( Button & button )
-{
-	bool state = !Config->get_clicking();
-	Config->set_clicking( state );
-	return state;
-}
-
-LedState MackieControlProtocol::clicking_release( Button & button )
-{
-	return Config->get_clicking();
-}
-
-LedState MackieControlProtocol::global_solo_press( Button & button )
-{
-	bool state = !session->soloing();
-	session->set_all_solo ( state );
-	return state;
-}
-
-LedState MackieControlProtocol::global_solo_release( Button & button )
-{
-	return session->soloing();
-}
-
 /////////////////////////////////////
 // Bank Switching
 /////////////////////////////////////
@@ -1447,3 +1500,50 @@ LedState MackieControlProtocol::marker_release( Button & button )
 {
 	return off;
 }
+
+void jog_wheel_state_display( JogWheel::State state, MackiePort & port )
+{
+	switch( state )
+	{
+		case JogWheel::zoom: port.write( builder.two_char_display( "Zm" ) ); break;
+		case JogWheel::scroll: port.write( builder.two_char_display( "Sc" ) ); break;
+		case JogWheel::scrub: port.write( builder.two_char_display( "Sb" ) ); break;
+		case JogWheel::shuttle: port.write( builder.two_char_display( "Sh" ) ); break;
+		case JogWheel::speed: port.write( builder.two_char_display( "Sp" ) ); break;
+		case JogWheel::select: port.write( builder.two_char_display( "Se" ) ); break;
+	}
+}
+
+Mackie::LedState MackieControlProtocol::zoom_press( Mackie::Button & )
+{
+	_jog_wheel.zoom_state_toggle();
+	update_global_button( "scrub", _jog_wheel.jog_wheel_state() == JogWheel::scrub );
+	jog_wheel_state_display( _jog_wheel.jog_wheel_state(), mcu_port() );
+	return _jog_wheel.jog_wheel_state() == JogWheel::zoom;
+}
+
+Mackie::LedState MackieControlProtocol::zoom_release( Mackie::Button & )
+{
+	return _jog_wheel.jog_wheel_state() == JogWheel::zoom;
+}
+
+Mackie::LedState MackieControlProtocol::scrub_press( Mackie::Button & )
+{
+	_jog_wheel.scrub_state_cycle();
+	update_global_button( "zoom", _jog_wheel.jog_wheel_state() == JogWheel::zoom );
+	jog_wheel_state_display( _jog_wheel.jog_wheel_state(), mcu_port() );
+	return
+		_jog_wheel.jog_wheel_state() == JogWheel::scrub
+		||
+		_jog_wheel.jog_wheel_state() == JogWheel::shuttle
+	;
+}
+
+Mackie::LedState MackieControlProtocol::scrub_release( Mackie::Button & )
+{
+	return
+		_jog_wheel.jog_wheel_state() == JogWheel::scrub
+		||
+		_jog_wheel.jog_wheel_state() == JogWheel::shuttle
+	;
+}
